'''
This is a "Hello World" example testing the response of four classes of
 neuron models to a single, shared, 400Hz Poisson spike source.

This demo tests

- each of the models in SpikeStream.neurons
- the response of said models to a shared poisson spike input

note that the automatically generated code for this demo is a bit
inefficient, but this is not a typical model structure
'''

#import the SpikeStream library
from SpikeStream import *

DT           = 0.01 #ms
POISSONRATE  = 400 #Hz
LIFTAU       = 5 #ms
LIFRESET     = -70 #mV
LIFTHRESH    = -55 #mV
LIFREFACTORY = 2 #ms
SYNAPSETAU   = 3 #ms
SYNAPSEREV   = -10 #mV
SYNAPSEDG    = 0.5 #mS ?
NTRIAL       = 1 #just one sim
NTDIV        = NTRIAL #we want to run all the replocas at once
TIME         = 200 #ms
STEPS        = int(TIME/DT) #time steps to simulate

#Set up simulation framework
sim = newSim(NTRIAL,NTDIV,False,DT)

#add a bunch of different types of neuron to the simulation
#you need to give the models different names so you dont have namespace
#collisions in the generated code
poisson_neuron             = sim.newModel(1,Poisson(POISSONRATE,name='Poisson'))
LIF_neuron                 = sim.newModel(1,LIF(LIFTAU,LIFRESET,LIFTHRESH,name='lif'))
LIF_refractory_neuron      = sim.newModel(1,LIF_Refractory(LIFTAU,LIFRESET,LIFTHRESH,LIFREFACTORY,name='lif_ref'))
izhikevich_tonic_spiking   = sim.newModel(1,Izhikevich(0,name='ts'))
izhikevich_phasic_spiking  = sim.newModel(1,Izhikevich(1,name='ps'))
izhikevich_tonic_bursting  = sim.newModel(1,Izhikevich(2,name='tb'))
izhikevich_phasic_bursting = sim.newModel(1,Izhikevich(3,name='pb'))

models = [LIF_neuron,LIF_refractory_neuron,izhikevich_tonic_spiking,izhikevich_phasic_spiking,izhikevich_tonic_bursting,izhikevich_phasic_bursting]

#define an excitatory synapse
e_synapse = ExponentialSynapse('e',SYNAPSETAU,SYNAPSEREV,SYNAPSEDG)

#tell the simulation to use this kind of synapse
#for all connections from poisson_neuron to the other neurons
#this both tells the poisson neuron to use this synapse
#and generates state variables in the target models for this synapse
for model in models:
    sim.setTransmitter(poisson_neuron,model,e_synapse)
    sim.connect_pair(poisson_neuron,0,model,0)

#add a probe to record voltage
voltage = [State(sim,neuron,'v') for neuron in models]
poisson_spikes = AllSpikesProbe(sim,poisson_neuron)

#execute the simulation
sim.prepare_and_run(STEPS)

#a quick plot
import re
import matplotlib as mpl
import matplotlib.pyplot as plt
textf = lambda s:'$\mathrm{%s}$'%re.sub(r" ", r"\\ ",s)
fig = plt.figure()
for i,model in enumerate(models):
    ax = fig.add_subplot(len(models),1,1+i)
    ax.plot(flatten(voltage[i].data))
    if model!=models[-1]: ax.set_xticklabels([])
    if model==models[0]: ax.set_title(textf('Voltage Traces for Several Models'))
ax.set_xlabel(textf('Timestep (dt=%sms)'%DT))
fig.show()
savefig_allformats(fig,"demo_models_plot")


