'''
Useful Reference Documentation
==============================

  for an overview of the Python language see
    http://docs.python.org/
    
  for an overview of numpy see
    http://mathesaurus.sourceforge.net/matlab-numpy.html
    http://docs.scipy.org/doc/

  for an overview of matplotlib see
    http://matplotlib.sourceforge.net/

  for an overview of PyCuda see
    http://documen.tician.de/pycuda/

  for an overview of iPython
    http://ipython.scipy.org/moin/Documentation

  for an overview of Sage
    http://www.sagemath.org/


Welcome to my heavily commented olfactory bulb model code !
===========================================================

This code, to some extent, requires learning how SpikeStream worked, 
which is a fair bit of effort, but I will summarize a bit here, but see the
spikestream.py file in the src directory for more information. 

Spike stream is designed for parallel execution of spiking neuron
simulations. 

Simulation : the main class, contained everyting
Model      : a type of neruon, e.g. LIF, EIF, Poisson
Probe      : an object for reading information out of the sim while it runs

you can run one large simulation in parallel. each neuron will get its own
processor, and it will be fast. you can also run multiple copies of the same
simulation with different parameters in parallel. this can be good for
parameter sweeps, gathering large samples, or testing different stimuli. 

If you want to run multiple copies of a network, listen up : 
- a single copy is called a relica in SpikeStream, or realization in AHH
- usually, not all the copies really fit on the graphics card at once, 
- they must be broken down into "Divisions", which are groups of replicas

so, the execution of several simulations in parallel will look like
- a collection of divisions, executed in sequence, each containing
- - a collection of replicas, each its own simulation, executed in parallel

most of my probes for collecting data don't do much to hide this mode of
operation. so, if you are following the voltages of all cells, you are not
going to get an array where you can do

voltages[sim_num][cell_num]

though that might be nice, it involved a lot of slow data re-arrangement.
what you actually get is

voltages[division_num][replica_within_division*cells_per_replica+cell_num]

so, thats less fun, but thats how the data comes out of the GPU. All the 
replicas within one division are treated as one gigantic simulation, so 
state is returned as a vector with all the replicas contained.

synapse variables are simulation-wide, while
state variables are specific to each model instance. So, if you have a 
population of 100 lif neurons called Foo, and you bind a voltage probe to
foo, you just get the voltage for the 100 units in Foo, unless you have
multiple replicas, then you get 100*number of replicas. 
'''

'''
Import time and start timing initialization for benchmark purposes

You can import packages with the import statement
>> import packagename
>> packagename.foo()

Once imported, functions and classes within a package can be referenced as
packagename.objectinpackage.

you can also assign a shortened version of the packagename for convenience
>> import packagename as pm
>> pm.foo()
'''
import time
t0 = time.time()

'''
the next two lines run the spikestream initialization code
spikestream home should be the correct path to the spikestream src directory
currently it is set relative to the directory containing this file
note that you will have to update this path if you move this file

you can also import packages using the
>>> from packagename import foo,bar,baz
where you specify which functions or classes you want to import.
you can reference things imported this way without typing the package name

you can also do
>> from packagename import *
which imports all objects in packagename and puts them in your current 
namespace, so you don't have to ever typepackagename.foo 
'''
from SpikeStream import *

import random
import numpy

'''
I am seeding all my random number generators for reproducability
hopefully having both of these have the same seed wont cause trouble
'''
random.seed(9876413)
numpy.random.seed(9876413)

'''
Simulation Parameters

This simulation is set up to simulate binding of a set of fictional odors
at a range of concentrations. 

It records and reports several statistics of the response of the network

I should note that there is hardly any such thing as "steady state" in
olfaction, since the respiratory rythm provides continual oscillations, 
so to some extend waiting for transients to die out is as wrong as 
keeping them per simulationulation, until we get realistic odor time courses.
'''

DT      = 0.2               # Time step in milliseconds
NODORS  = 1                 # number of simulated odors to present
INCS    = 1                 # number of concentrations at which to test an odor
NRECEPT = 50                # number of olfactory receptor types
LOBOUND = -4                # log10 lower odor concentration
HIBOUND = 4                 # log10 upper odor concentration
NOGLOM  = 20                # number of ORNs per glomerulus
NMGLOM  = 50                # number of mitral cells per glomerulus
NGM     = 5                 # number of granule cells per mitral Cell
NGSYN   = 50                # Number of synapses per granule Cell
NOTARG  = int(0.5*NMGLOM)   # number of ORN-mitral synapses per ORN
ORNRATE = 200.0             # The max rate of ORNs at 100% bound receptor   Hz
GRATE   = 1.0/20.0          # The rate scalar for granule cell           Hz/uS
OM_DG   = 5.0               # conductance change for the ORN-mitral synapse uS
MG_DG   = 0.1               # conductance change for mitral-granule synapse uS
GM_DG   = 0.1               # conductance chance for granule-mitral synapse uS
OM_REV  = -10.0             # net reversal potential for ORN -> MC synapses mV
GM_REV  = -70.0             # net reversal potential for GC -> MC synapses  mV
OM_TAU  = 5.0               # time constant for ORN -> MC synapse           ms
GM_TAU  = 5.0               # time constant for GC -> MC synapse            ms
MG_TAU  = 200.0             # time constance for MC -> GC synapse           ms
G_SMOOTH= 3                 # degree of alpha function for MC -> GC synapse
NM      = NMGLOM * NRECEPT  # total number of mitral cells per simulation
NG      = NGM * NM          # total number of granule cells per simulation
NORN    = NRECEPT * NOGLOM  # total number of ORNs per simulation
NUNITS  = NRECEPT + NM + NG # total number of neurons per simulation
NTRIAL  = NODORS*INCS       # total number of conditions to test
NTDIV   = NTRIAL            # try to fit all trials in one division
DSTIM   = int(2000/DT)        # number of time steps to simulate
SMOOTH  = 2                 # degree of alpha function for MC synapsess

# flags for what to recors 1=True=do record, 0=False=don't record
RECORD_GLOM_SPIKES     = 0
RECORD_MITRAL_CURRENT  = 1
RECORD_ORN_SPIKES      = 0
RECORD_MITRAL_SPIKES   = 1
RECORD_GRANULE_SPIKES  = 0
RECORD_MC_MEAN_VOLTAGE = 1
RECORD_MC_CONDUCTANCES = 1

'''
This is... the rather hackish way to get input into the ORNs. You could also
give each ORN a state variable representing the rate, and then initialize
the rate variables across all cells per simulationulation. 

This way, we prepare the rate information, upload it to the GPU, then
explicitely reference it in the ORN model. "Brittle" is a good way to 
describe the approach

1 : precompute a range of concentrations for each odor
2 : for each odor, generate a log-normal distribution of receptor affinities
3 : for each odor,concentration, compute fraction bound using the hill eq.
4 : flatten this matrix and copy it to the GPU

Later on, we can access this information using the absolutely lovely 
expression.

reinterpret_cast<float*>(device_pointer_as_integer)

Device pointers can be had from instances of PyCuda GPUArray my casting
an instance of gpuarray to int : int(gpuaray_instance)
'''
if INCS>1:
    logconc = [LOBOUND+float(i)/(INCS-1)*(HIBOUND-LOBOUND) for i in xrange(INCS)]
else:
    logconc = [0.5*(LOBOUND+HIBOUND)]
pkd_matrix     = [normal(0,1)(NRECEPT) for i in xrange(0,NODORS)]
hill           = lambda pkd:(lambda b:b/(b+10**-pkd))
binding_matrix = [[[hill(pKd)(10**logconc[i]) for pKd in row] for row in pkd_matrix] for i in xrange(INCS)]
d_bound        = cuda.to_device(numpy.array(list(flatten(flatten(binding_matrix))),numpy.float32))
               

'''
Here I define the different point-neuron models used per simulationulation.
To definine a new type of neuron, you must extend the Model class, which is
defined in Spikestream. "class new_neuron_type(Model):"

you should also define an __init__ function that will initialize the neuron
"   def __init__(self,this,is_a,list_of,arguments):"

Every model has a name. This is the name used in code-generation, so, when
you make an instance of a neuron model, you should give that population of
neurons a unique name. In your customized model class, you should call the
model initializer with the name of the model "Model.__init__(self,name)".
"       Model.__init__(self,name)"

Neuron models have state variables. I think there is some sort of bug in
SpikeStream that requires each neuron to have a voltage variable. This 
requirement is actually quite reasonable, since otherwise the voltage gated
channels wouldn't work. 

I have written a little helper function for adding new state variables called
newVar. newVar needs to be called with "self" as the first argument. It 
accepts a variable name, an update euqation, and a function for performing
initialization.

All neurons need a spike_condition. This is set to a string, which 
will be inserted into the generated code. That string should specify a
boolean expression of the model state variables. For instance, 'v>0' would 
be a good spike_condition for a cell with a spiking threshold of 0mV.

you can also specify code to get executed when a cells spikes verses when a 
cell does not spike. At the moment, this is mostly used to reset model
state variable in the event of a spike.
,d_bound
Channels and synpases onto neurons are modeled independently.
'''

class ORN(Model):
    '''
    This is my olfactory receptor neron model. Its not complicated, just a
    simple inhomogenous poisson spike generator. 
    
    ok, actually, we do something funny here where we initialized the odor 
    information on the GPU separately in an array called d_bound, and then
    hard code the location of this stimulus information in the neuron class. 
    
    This is an inelegent solution and is one of the reasons Cyrus has tried
    to write a mode flexible incarnation of SpikeStream in AHH. 
    
    Another way to do this would be to make a state variable called "rate",
    and to upload new information to this rate variable over the 
    course of the simulation.
    '''
    def __init__(self,maxrate=0,name="ORN"):
        Model.__init__(self,name)
        self.spike_condition = 'RAND<%f*DT*(reinterpret_cast<float*>(%d)[replica_num*%d+idx_model/%d])'%(maxrate,d_bound,NRECEPT,NOGLOM)

class mitralCellMkII(Model):
    '''
    Here is my mitral cell model. This is taken directly out of Izhekeviche's
    dynamics of excitability and bursting text.s
    '''
    def __init__(self,binit,name="mitralCellMkII"):
        Model.__init__(self, name)
        self.newVar('vd',deqn('vd','0.01235*(v-vd)'),constant(-65))
        self.newVar('v',deqn('v','0.025*((v+55.0)*(v+50.0)+0.5*(vd-v)-u+input_current)','reset'),constant(-65))
        self.newVar('u',deqn('u','0.4*((v<-48?0:20*(v+48.0))-u)'),constant(-8))
        self.spike_condition = 'v>=35.0'
        self.reset = "d_v[idx_state] = -50;"
        self.dont_reset = "d_v[idx_state] = new_v;"
        
class granuleCell(Model):
    '''
    Here is my granule cell model. This model is a bit of a hack. 
    
    Rather than computing cell voltage, and simulating a spiking mechanism, 
    I use the conductance of the incoming synapse model to modeluate the 
    rate of an inhomogenous poisson process.
    
    If the incoming synapse is modeled as a slow tau~=100ms alpha function
    synapse, the conductance as a result of a spike can be scaled to a 
    cell firing rate. This captures the slow rise time and asynchronous
    release aspects of granule cell function. So, this relies on using
    a slow incoming synapse with the granule cells model.
    
    Since Spikestream will still try to automatically model cell currents
    through the incoming synapses, I need to give it a fake voltage variable
    
    I guess that is kind of a bug.
    
    also, this relies on Mitral->Granule synapse being a 3rd order
    alpha like synapse named "MG". bad style.
    '''
    def __init__(self,name="granuleCell"):
        Model.__init__(self,name)
        self.newVar('v','')#TODO MAKE THIS NOT NECESSARY !
        self.spike_condition = 'RAND<g_2_MG*DT*%sf'%float(GRATE)    

class mitralProbe(Probe):
    # WARNING REQUIRES INTEGRAL DIVISION INTO CHUNKS
    '''
    Here is an example probe.
    This probe has been designed to sum together all spikes from
    within the same glomerulus, to measure a course picture of the
    mitral cell response.
    
    See SpikeStream.py for the implementation and documentation of the probe
    class, and information on writing your own probe. I have defined some
    probes in SpikeStrea-extendedAPI.py,
    and cyrus has defined some probes in /src/probes directory
    do not run the simulation more than once if using cyrus' probes.
    '''
    def __init__(self,sim,model,chunks):
        Probe.__init__(self,sim)
        self.model = model
        self.chunks = chunks
    def get_device_memory_needed(self):
        return 4*self.info.count*self.sim.replicas_per_division    
    def allocate_memory(self):
        self.info      = self.sim.get_model_info(self.model)
        self.length    = self.info.count*self.sim.replicas_per_division
        self.chunksize = self.info.count/self.chunks
        self.memory    = pycuda.gpuarray.to_gpu(numpy.zeros(self.length).astype(numpy.int32))
        self.basep     = int(self.memory.gpudata)
        irange         = xrange(self.sim.replicas_per_division)
        self.arrays    =  [[gpuarray.GPUArray((self.chunksize,),numpy.int32,\
                          base=self.memory,gpudata=self.basep+\
                          4*(i*self.info.count+j*self.chunksize)) \
                          for j in xrange(self.chunks)] for i in irange]
        self.data      = [[] for i in xrange(self.sim.num_divisions)]
    def release_memory(self):
        del self.info
        del self.arrays
        del self.memory
    def generate_device_code(self):
        self.spike_code = [([self.model],'reinterpret_cast<int*>(%d)[idx_state]+=1;'%(self.memory.gpudata))]
    def prepare_division(self, first_idx_sim, num_replicas_processing):
        self.division = first_idx_sim / sim.replicas_per_division
        cuda.memset_d32(self.basep,0,self.length)
    def division_complete(self, first_idx_sim, num_replicas_processed):
        batch = []
        for replica in self.arrays:
            mod = []
            for glomerulus in replica:
                temp = pycuda.gpuarray.sum(glomerulus,numpy.int32)
                mod.append(int(temp.get()))
            batch.append(mod)
        self.data[self.division].append(batch)

'''
Make a new simulation
'''
sim     = newSim(NTRIAL,NTDIV,False,DT)

'''
Create new models, add them to the simulations
'''
orn     = sim.newModel(NORN,ORN(ORNRATE/1000.0))
mitral  = sim.newModel(NM,mitralCellMkII(uniform(0,1)))
granule = sim.newModel(NG,granuleCell())

'''
Set up transmitters between populations

the convention is
simulation, from, to, type of synapse

note that the reversal potential for the MC-GCs synapse is not used, since
firing rate is computed directly from the conductance variable, 
since I lacked an electrophisiological model of a granule cell.
'''
sim.setTransmitter(orn,mitral,DampedSynapse('OM',SMOOTH,OM_TAU,OM_REV,delta_g=OM_DG))
sim.setTransmitter(granule,mitral,DampedSynapse('GM',SMOOTH,GM_TAU,GM_REV,delta_g=GM_DG))
sim.setTransmitter(mitral,granule,DampedSynapse('MG',G_SMOOTH,MG_TAU,0.0,delta_g=MG_DG))

'''
Build a connectivity matrix ( add in synapses )
'''
for i in xrange(0,NRECEPT):
    sim.connect_random_subset(orn,xrange(i*NOGLOM,(i+1)*NOGLOM),mitral,xrange(i*NMGLOM,(i+1)*NMGLOM),NOTARG)
sim.connect_random_subset(granule,xrange(0,NG),mitral,xrange(0,NM),NGSYN,symmetric='true')

'''
Add probes to record simulation state. probes slow down execution.

Probes :

SpikeStream handles pulling data out of ( and, theoretically, putting data 
into ) a running simulation with these objects called probes. Probes get 
added to the simulation, 
and certain functions in the probe get called at certain phases of simulation
execution. However, it is not straightforeward as "this gets called
every time step". You must write a function that returns the 
amount of memory used, a function that allocates memory, a function that 
releases memory, a function that returns code to run on the GPU, a
function that gets called before something called a division runs ( a 
division is, I think, a group of simulations, in case you want to run 
multiple simulations in parallel ), and another function to be called when
a division is complete, you can also override a function for each timestep
of the simulation, but remember that this will only apply to all replicas
within a division. 

See the definition of Probe in SpikeStream.py

So, hopefully I will write up all the ideosyncracies of probes some other
time.

Note that you can use Cyrus' probes as well. You may like these better. 
However, Cyrus's probes only work once : if you try to run the 
simulation more than once I think things crash. My probes can handle
multiple launches of the simulation but are less sophisticated.
'''

if RECORD_MC_MEAN_VOLTAGE:
    mean_mitral_v = Field(sim,mitral,'v')

'''
counts the number of spikes from each glomerulus over the course of the
simulation.
'''
if RECORD_GLOM_SPIKES:
    glom_spikes = mitralProbe(sim,mitral,NRECEPT)

'''
each neuron model also has an automatically generated input_current var
this is a hack to get mean population current.
you can also use this to pull out other automatically generated state, 
though I think the "Expression" probe will achieve the same effect.
Maybe Cyrus has somehting for this.
'''
if RECORD_MITRAL_CURRENT:
    mitral.newVar('current','current=input_current;')
    mean_mitral_i      = Field(sim,mitral,'current')

'''
we can also take a look at conductances as follows
'''
if RECORD_MC_CONDUCTANCES:
    mitral.newVar('g_e_monitor','g_e_monitor=g_%d_OM;'%(SMOOTH-1))
    mitral.newVar('g_i_monitor','g_i_monitor=g_%d_GM;'%(SMOOTH-1))
    mean_mitral_exc = Field(sim,mitral,'g_e_monitor')
    mean_mitral_inh = Field(sim,mitral,'g_i_monitor')
    
'''
The next three lines can log spikes. These store spike data in a naive 
format, which is just a matrix of 1 if a cell spikes at time t else 0
these are memory and time intensive so I have turned them off
'''
if RECORD_ORN_SPIKES:
    all_orn_spikes     = AllSpikesProbe(sim,orn)
if RECORD_MITRAL_SPIKES:
    all_mitral_spikes  = AllSpikesProbe(sim,mitral)
if RECORD_GRANULE_SPIKES:
    all_granule_spikes = AllSpikesProbe(sim,granule)

'''
you must prepare the simulation. This step allocates memory and compiles
the code
'''
sim.prepare()

'''
this performs the actual computation. it may take a while
I've wrapped these calls in a timer function for benchmarking purposes
'''
prep_time = time.time()-t0
t0 = time.time()
sim.run(DSTIM)
sim_time = time.time()-t0

'''
Now time for some analysis code
'''
#this regular expression makes my titles into LaTeX "equations"
#which I pretty much only use in order to get a +serif style font
# edit : apparantly it doesn't like the '#' symbol
mathlike = lambda s:'$\mathrm{%s}$'%re.sub(r" ", r"\\ ",s)

if RECORD_MC_MEAN_VOLTAGE:
    '''
    for the Field probe, the data array is divided as follows : 
    division
    timestep
    replica
    
    the zip(*foo) idiom is one way of saying "matric transpose" when 
    working on standard Python arrays. If working on numpy arrays, see
    the numpy documentation. Other syntax may be faster
    
    Here, zip(*mean_mitral_v.data[0]) takes an matrix where each row is a
    time-step, and changes it to an array where each row is the time series
    for a given replica.
    '''
    mitral_cell_mean_voltage = zip(*mean_mitral_v.data[0])
    fig,ax = newplot(mitral_cell_mean_voltage,"timestep, dt=%s"%DT,"mean voltage (mV)","Average Mitral Cell Voltage")
    savefig_allformats(fig,"mitral_cell_mean_voltage")

if RECORD_MITRAL_CURRENT:
    mitral_cell_mean_current =  zip(*mean_mitral_i.data[0])
    fig,ax = newplot(mitral_cell_mean_current,"timestep, dt=%s"%DT,"mean current (nA)","Average Mitral Cell Current")
    savefig_allformats(fig,"mitral_cell_mean_current")

if RECORD_ORN_SPIKES:
    '''
    The AllSpikesProbe stores data in a 4D array as follows :
    - which call to sim.run ? ( we only have 1 )
    - which division ( we only have 1 )
    - which time-step
    - which cell ( all replicas in division are present, you must cut them
      apart to get individual replicas separately )
    '''
    orn_spikes = all_orn_spikes.alldata[0][0]
    
if RECORD_MITRAL_SPIKES:
    mitral_spikes = all_mitral_spikes.alldata[0][0]
    '''
    example spike raster scatter-plot ( single odor demos only ? )
    -- transpose list of spikes to get a per-cel representation
    -- convert the spike raster to a set of time points
       in ahh this will be GPU accelerated, its on the CPU for now
    -- prepare scatter plot, colored by receptor
       preparing each set of glom spikes is much faster
       than plotting one cell at a time
    
    I feel like cyrus probably has some GPU code to speed this plotting process
    up. Binning spikes is rather slow.
    '''
    mitral_cell_spikes = zip(*mitral_spikes)
    
    spike_times = []
    for cell_id in xrange(len(mitral_cell_spikes)):
        raster = mitral_cell_spikes[cell_id]
        temp = []
        for i in xrange(len(raster)):
            if raster[i]:
                temp.append(i*DT)
        spike_times.append(temp)
            
    if NODORS==1 and INCS==1: 
        print "Please wait, generating mitral cell spike raster"
        
        fig = plt.figure()
        ax = fig.add_subplot(111)
        cell_id=0
        color_cycle="rcgmbyk"
        for glom_id in xrange(NRECEPT):
            temp_id    = []
            temp_spike = []
            for cell_id in xrange(glom_id*NMGLOM,(glom_id+1)*NMGLOM):
                spikes = spike_times[cell_id]
                temp_id.extend([cell_id]*len(spikes))
                temp_spike.extend(spikes)
            ax.plot(temp_spike,temp_id,'%s.'%color_cycle[glom_id%len(color_cycle)],markersize=0.5)

        ax.set_title(mathlike('Raster of Mitral Cell Spikes Colored by Glomerulus'))
        ax.set_xlabel(mathlike("Time (ms)"))
        ax.set_ylabel(mathlike("Cell ID number"))
        fig.show()
        savefig_allformats(fig,"MC raster")
    
    '''
    compute mitral cell firing rate disribution
    -- compute rates
    -- compute histogram
    -- pool rates amongst glomeruli
    -- plot rate transfer function of fraction bound for each receptor
    '''
    norm_factor = ((DSTIM+1)*DT*0.001)
    rates = [ float(len(raster))/norm_factor for raster in spike_times ]
    fig,ax = histplot(rates,20,'Mitral Cell Firing Rate (Hz)','p(Rate)','Histogram of Mitral Cell Firing Rates')
    savefig_allformats(fig,"MC histogram")
    
    '''
    TODO
    compute synchronization between glomeruli
    -- bin spike raster down to some "reasonable" timescale ( 1-2 spikes/bin )
    -- box convolve with "timescale of interest" which is probably 10ms
       maybe 100ms
    -- compute zero lag pair-wise correlation
    -- mean zero lag pairwise = "measure of synchrony"
    '''
    
if RECORD_GRANULE_SPIKES:
    granule_spikes = all_granule_spikes.alldata[0][0]

if RECORD_GLOM_SPIKES:
    total_glomerulus_spikes = glom_spikes.data[0][0]
    '''
    TODO
    compute input-output correlations from this data
    -- compute pairwise correlations in fraction bound
    -- compute pairwise correlations in Glomerular vectors
    -- plot the correlation transfer function nicely
    -- plot correlation transfer function using rate nonlinearity ?
    -- plot correlation transfer function using linear inhibition ?
    '''

if RECORD_MC_CONDUCTANCES:
    '''
    I'm not sure this plotting script will handle multiple odors in a 
    single simulation gracefully.
    '''
    mitral_cell_mean_e = zip(*mean_mitral_exc.data[0])[0]
    mitral_cell_mean_i = zip(*mean_mitral_inh.data[0])[0]
    if NODORS==1 and INCS==1: 
        print "Generatig mean conductance plot"
        #compliated plotting script, see the matplotlib docs
        mathlike = lambda s:'$\mathrm{%s}$'%re.sub(r" ", r"\\ ",s)
        fig = plt.figure()
        ax = fig.add_subplot(111)
        ax.plot(mitral_cell_mean_e,'b-')
        ax.plot(mitral_cell_mean_i,'r-')
        leg = ax.legend((mathlike('Excitatory Conductance from ORNs'), mathlike('Inhibitory Conductance form Granule Cells')),'best', shadow=False)
        ax.set_title(mathlike("Average Mitral Cell Conductances"))
        ax.set_xlabel(mathlike("timestep, dt=%s"%DT))
        ax.set_ylabel(mathlike("mean conductance (uS)"))
        fig.show()
        savefig_allformats(fig,"ei conductance")

'''
Demonstration of how to save data as a matlab file
1 : make sure you import the scipy.io library
2 : create an empty dictonary {}
3 : assign values to words in the dictionary
4 : use scipy.io to save dictionary as a .mat file
5 : loading .mat file in Matlab puts saved python objects in workspace

In MATLAB
>> load mydata.mat
If you would like to open a matlab file as a dictionary, use scipy.io.loadmat
This will create a dictionary somedata containing the matlab workspace saved
in the .mat file. You can then extract objects from the dictionary.
>> somedata = scipy.io.loadmat('data.mat')
>> glom = somedata['total_glomerulus_spikes']
'''
import scipy.io
mydata = {} 

mydata['pkd_matrix']=pkd_matrix
mydata['binding_matrix']=binding_matrix

if RECORD_MC_MEAN_VOLTAGE:
    mydata["mitral_cell_mean_voltage"] = mitral_cell_mean_voltage 
if RECORD_MITRAL_CURRENT:
    mydata["mitral_cell_mean_current"] = mitral_cell_mean_current
if RECORD_ORN_SPIKES:
    mydata["orn_spikes"] = orn_spikes
if RECORD_MITRAL_SPIKES:
    mydata["mitral_spikes"] = mitral_spikes 
if RECORD_GRANULE_SPIKES:
    mydata["granule_spikes"] = granule_spikes 
if RECORD_GLOM_SPIKES:
    mydata["total_glomerulus_spikes"] = total_glomerulus_spikes
if RECORD_MC_CONDUCTANCES:
    mydata["mitral_cell_mean_e"] = mitral_cell_mean_e 
    mydata["mitral_cell_mean_i"] = mitral_cell_mean_i    
scipy.io.savemat('data.mat',mydata)


'''
Print out a summary of simulation parameters as well as runtime stats
'''
summary = ""
summary += "Simulation summary"+"\n"
summary += "=================="+"\n"
summary += "Physical Parameters ( minimal effect on runtime )"+"\n"
summary += " log10 lower odor concentration : %s"% LOBOUND+"\n"
summary += " log10 upper odor concentration : %s"% HIBOUND+"\n"
summary += " the maximum rate of ORNs at 100%% bound receptor (Hz) : %s"% ORNRATE+"\n"
summary += " the rate scalar for granule cell : %s (Hz/uS)"% GRATE+"\n"
summary += " conductance change for the ORN-mitral synapse (uS) : %s"% OM_DG+"\n"
summary += " conductance change for mitral-granule synapse (uS) : %s"% MG_DG+"\n"
summary += " conductance chance for granule-mitral synapse (uS) : %s"% GM_DG+"\n"
summary += " net reversal potential for ORN -> MC synapses (mV) : %s"% OM_REV+"\n"
summary += " net reversal potential for GC -> MC synapses (mV) : %s"% GM_REV+"\n"
summary += " time constant for ORN -> MC synapse (ms) : %s"% OM_TAU+"\n"
summary += " time constant for GC -> MC synapse (ms) : %s"% GM_TAU+"\n"
summary += " time constance for MC -> GC synapse (ms) : %s"% MG_TAU+"\n"
summary += " degree of alpha function for GC synapses : %s"% G_SMOOTH+"\n"
summary += " degree of alpha function for MC synapses : %s"% SMOOTH+"\n"
summary += ""+"\n"
summary += "Network Parameters ( contribute to runtime complexity )"+"\n"
summary += " time step (ms) : %s"%DT+"\n"
summary += " number of simulated odors to present : %s"% NODORS+"\n"
summary += " number of concentrations at which to test an odor : %s"% INCS+"\n"
summary += " number of olfactory receptor types : %s"% NRECEPT+"\n"
summary += " number of ORNs per glomerulus : %s"% NOGLOM+"\n"
summary += " number of mitral cells per glomerulus : %s"% NMGLOM+"\n"
summary += " number of granule cells per mitral Cell : %s"% NGM+"\n"
summary += " number of synapses per granule Cell : %s"% NGSYN+"\n"
summary += " number of ORN-mitral synapses per ORN : %s"% NOTARG+"\n"
summary += " total number of ORNs per simulation : %s"% NORN+"\n"
summary += " total number of mitral cells per simulation : %s"% NM+"\n"
summary += " total number of granule cells per simulation : %s"% NG+"\n"
summary += " total number of neurons per simulation : %s"% NUNITS+"\n"
summary += " total number of conditions to test : %s"% NTRIAL+"\n"
summary += " number of time steps to simulate : %s"% DSTIM+"\n"
summary += ""+"\n"
summary += "Data Stored ( copied while simulation running )"+"\n"
summary += " glomerulus spike counts : %s"%RECORD_GLOM_SPIKES+"\n"
summary += " mean current in mitral cell layer : %s"%RECORD_MITRAL_CURRENT+"\n"
summary += " all ORN spikes : %s"%RECORD_ORN_SPIKES+"\n"
summary += " all mitral layer spikes : %s"%RECORD_MITRAL_SPIKES+"\n"
summary += " all granule layer spikes : %s"%RECORD_GRANULE_SPIKES+"\n"
summary += " mean mitral cell voltage : %s"%RECORD_MC_MEAN_VOLTAGE+"\n"
summary += " mitral cell e,i input conductances : %s"%RECORD_MC_CONDUCTANCES+"\n"
summary += ""+"\n"
summary += "Runtime Summary"+"\n"
summary += " total number of units simulated in parallel : %s"%(NUNITS*NTRIAL)+"\n"
summary += " total time for python side initialization (s) : %s"%prep_time+"\n"
summary += " total time for simulation run (s) : %s"%sim_time+"\n"
summary += " frames per second : %s"%(DSTIM/sim_time)+"\n"
summary += " per-cent realtime : %s%%"%(100.0*(DSTIM*DT/1000.0)/sim_time)+"\n"
summary += ""
print summary
f = open('summary.txt', 'w')
f.write(summary)
f.flush()
f.close()





